import { Injectable } from '@angular/core';
import {
  ActivatedRouteSnapshot,
  Resolve,
  Router,
  RouterStateSnapshot,
} from '@angular/router';
import { catchError, Observable, throwError } from 'rxjs';

import { Recommendation } from '../models/recommendation';
import { RecommendationPagination } from '../models/recommendation-pagination';
import { RecommendationService } from '../services/recommendation.service';

@Injectable({
  providedIn: 'root',
})
export class RecommendationResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(
    private _recommendationServiceService: RecommendationService,
    private _router: Router
  ) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<Recommendation | undefined> {
    return this._recommendationServiceService
      .getRecommendationById(route.paramMap.get('id'))
      .pipe(
        // Error here means the requested product is not available
        catchError((error) => {
          // Log the error
          console.error(error);

          // Get the parent url
          const parentUrl = state.url.split('/').slice(0, -1).join('/');

          // Navigate to there
          this._router.navigateByUrl(parentUrl);

          // Throw an error
          return throwError(error);
        })
      );
  }
}

@Injectable({
  providedIn: 'root',
})
export class RecommendationsResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(private _recommendationService: RecommendationService) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<{
    pagination: RecommendationPagination;
    recommendations: Recommendation[];
  }> {
    return this._recommendationService.getRecommendations();
  }
}
